'Program komunikujcy si z pamicia AT93C46 (1k - 128*8) - pojemno 128 bajtw
'Komunikacja z pamiecia odbywa sie przy pomocy magistrali SPI
'Mikrokontroler ATMEGA8 taktowany czestotliwoscia 8MHz
'W programie do pamieci (komorki 0...127) wpisywane sa wartosci liczbowe od 0 do 127,
'nastepnie dane zapisane do pamieci sa odczytywane i weryfikowane pod wzgledem poprawnosci zapisu
'zarowno wartosci danych zapisywanych jak i odczytywanych sa wysylane do terminala poprzez interfejs RS232.
'Predkosc komunikacji przez RS232 wynosi 9600 bodow.
'W przypadku bledu zapisu do pamieci wysylany jest do terminala stosowny komunikat
'Marcin Wizania
'marcin.wiazania@ep.com.pl

$regfile = "m8def.dat"                                      'informuje kompilator o pliku dyrektyw mikrokontrolera
$crystal = 8000000                                          'informuje kompilator o czstotliwoci oscylatora taktujacego mikrokontroler
$baud = 9600                                                'informuje o predkosci transmisji interfejsu RS232

Config Pinb.5 = Output                                      'linia PB5 jako wyjsciowa
Config Pinb.2 = Output                                      'linia PB2 jako wyjsciowa
Config Pinb.3 = Output                                      'linia PB3 jako wyjsciowa

Declare Sub Zap_eeprom(byval Adres As Byte , Byval Dana As Byte)       'procedura zapisu danej do pamieci
Declare Sub Odc_eeprom(byval Adres As Byte , Dana As Byte)  'procedura odczytu danej z pamieci
Declare Sub Zap_en                                          'procedura odblokowania kasowania oraz zapisu pamieci
Declare Sub Zap_bl                                          'procedura zablokowania kasowania oraz zapisu pamieci
Declare Sub Res_pam                                         'procedura zerowania pamieci
Declare Sub Opozn_zap                                       'procedura opoznienia potrzebnego na czas zapisu do pamieci

Dim Adres As Byte                                           'zmienna adresu komorki pamieci
Dim Dana As Byte                                            'zmienna zmienna na odczytana lub zapisana dana do pamieci
Dim I As Byte                                               'zmienna licznikowa
Dim Temp As Byte                                            'zmienna pomocnicza

Sk Alias Portb.5                                            'przypisanie nazwie Portb.5 aliasu Sk
Cs Alias Portb.2                                            'przypisanie nazwie Portb.2 aliasu Cs
Od Alias Portb.3                                            'przypisanie nazwie Portb.3 aliasu Od
Di Alias Pinb.3                                             'przypisanie nazwie Pinb.3 aliasu Di

Reset Cs                                                    'zerowanie linii Cs
Reset Sk                                                    'zerowanie linii Sk
Set Od                                                      'ustawienie linii Od (linia DO)

Call Zap_en                                                 'odblokowanie mozliwosci kasowania oraz zapisu do pamieci
Call Res_pam                                                'czyszczenie zawartosci pamieci
Print "Zapis"                                               'wyslanie do terminala tekstu

For I = 0 To 127                                            'petla wykonywana dla I=0 do I=255
  Call Zap_eeprom(i , I)                                    'zapis do EEPROM wartosci zmiennej I pod adres wskazywany przez I
  Print Hex(i) ; " ";                                       'wyslanie do terminala wartosci zapisywanej do EEPROM w postaci szesnastkowej
Next I                                                      'zwiekszenie o 1 wartosci I

Print                                                       'wyslanie do terminala wiersza odstepu
Print "Odczyt i weryfikacja"                                'wyslanie do terminala tekstu

For I = 0 To 127                                            'petla wykonywana dla I=0 do I=255
  Call Odc_eeprom(i , Dana)                                 'odczytanie z EEPROM wartosc z komorki o adresie wskazywanym przez I
  If Dana = I Then                                          '--weryfikacja danych-- jesli Dana = I to
    Print Hex(dana) ; " ";                                  'wyslanie do terminala odczytanej wartosci z pamieci w postaci szesnastkowej
  Else                                                      'w przeciwnym razie
    Print "Dane nieprawidlowe"                              'wyslanie do terminala tekstu
    Exit For                                                'opuszczenie petli For
  End If
Next I                                                      'zwiekszenie o 1 wartosci I

Call Zap_bl                                                 'zablokowanie mozliwosci kasowania oraz zapisu do pamieci
End                                                         'koniec programu

Sub Opozn_zap                                               'procedura opoznienia potrzebnego na czas zapisu do pamieci
  Set Od                                                    'ustawienie linii Do
  Set Cs                                                    'ustawienie linii Cs
  Waitms 12                                                 'opoznienie 12 ms
  Reset Cs                                                  'zerowanie linii Cs
End Sub                                                     'koniec procedury

Sub Res_pam                                                 'procedura zerowania pamieci
  Set Cs                                                    'ustawienie linii Cs
  Set Od                                                    'ustawienie linii Do
  Temp = &B10000000                                         'kod rozkazu zerowania pamieci
  Shiftout Od , Sk , Temp , 1 , 3 , 1                       'wyslanie 3 bitow zmiennej Temp do pamieci
  Temp = &B10000000                                         'dalszy kod rozkazu zerowania pamieci
  Shiftout Od , Sk , Temp , 1 , 7 , 1                       'wyslanie 7 bitow zmiennej Temp do pamieci
  Reset Cs                                                  'zerowanie linii Cs
  Call Opozn_zap                                            'wywolanie procedury opoznienia
End Sub                                                     'koniec procedury

Sub Zap_en                                                  'procedura odblokowania kasowania oraz zapisu pamieci
  Set Cs                                                    'ustawienie linii Cs
  Set Od                                                    'ustawienie linii Do
  Temp = &B10000000                                         'kod rozkazu odblokowania zerowania i zapisu do pamieci
  Shiftout Od , Sk , Temp , 1 , 3 , 1                       'wyslanie 3 bitow zmiennej Temp do pamieci
  Temp = &B11000000                                         'dalszy kod rozkazu odblokowania zerowania i zapisu do pamieci
  Shiftout Od , Sk , Temp , 1 , 7 , 1                       'wyslanie 7 bitow zmiennej Temp do pamieci
  Reset Cs                                                  'zerowanie linii Cs
End Sub                                                     'koniec procedury

Sub Zap_bl                                                  'procedura blokowania kasowania oraz zapisu pamieci
  Set Cs                                                    'ustawienie linii Cs
  Set Od                                                    'ustawienie linii Do
  Temp = &B10000000                                         'kod rozkazu blokowania zerowania i zapisu do pamieci
  Shiftout Od , Sk , Temp , 1 , 3 , 1                       'wyslanie 3 bitow zmiennej Temp do pamieci
  Temp = &B00000000                                         'dalszy kod rozkazu blokowania zerowania i zapisu do pamieci
  Shiftout Od , Sk , Temp , 1 , 7 , 1                       'wyslanie 7 bitow zmiennej Temp do pamieci
  Reset Cs                                                  'zerowanie linii Cs
End Sub                                                     'koniec procedury

Sub Zap_eeprom(byval Adres As Byte , Byval Dana As Byte)    'procedura zapisu danej do EEPROM
  Temp = &B10100000                                         'kod rozkazu zapisu danej do pamieci
  Set Cs                                                    'ustawienie linii Cs
  Set Od                                                    'ustawienie linii Do
  Shiftout Od , Sk , Temp , 1 , 3 , 1                       'wyslanie 3 bitow zmiennej Temp do pamieci
  Shift Adres , Left , 1                                    'przesuniecie bitow zmiennej Adres o jedna pozycje w lewo
  Shiftout Od , Sk , Adres , 1 , 7 , 1                      'wyslanie 7 bitow zmiennej Adres do pamieci
  Shiftout Od , Sk , Dana , 1 , 8 , 1                       'wyslanie zmiennej Dana do pamieci
  Reset Cs                                                  'zerowanie linii Cs
  Call Opozn_zap                                            'wywolanie procedury opoznienia
End Sub                                                     'koniec procedury

Sub Odc_eeprom(byval Adres As Byte , Dana As Byte)          'procedura odczytu danej do EEPROM
  Temp = &B11000000                                         'kod rozkazu odczytu danej z pamieci
  Set Cs                                                    'ustawienie linii Cs
  Set Od                                                    'ustawienie linii Do
  Shiftout Od , Sk , Temp , 1 , 3 , 1                       'wyslanie 3 bitow zmiennej Temp do pamieci
  Shift Adres , Left , 1                                    'przesuniecie bitow zmiennej Adres o jedna pozycje w lewo
  Shiftout Od , Sk , Adres , 1 , 7 , 1                      'wyslanie 7 bitow zmiennej Adres do pamieci
  Reset Ddrb.4                                              'ustawienie linii PB4 jako wejscie
  Shiftin Di , Sk , Dana , 0 , 8 , 1                        'odczyt bajta danych z pamieci do zmiennej Dana
  Reset Cs                                                  'zerowanie linii Cs
  Set Ddrb.4                                                'ustawienie linii PB4 jako wyjscie
  Set Od                                                    'ustawienie linii Do
End Sub                                                     'koniec procedury